@@ -2,7 +2,7 @@ use std::iter::FromIterator;
 
 use crate::key::Key;
 use crate::repr::Decor;
-use crate::table::{Iter, IterMut, KeyValuePairs, TableKeyValue, TableLike};
+use crate::table::{Iter, IterMut, KeyValuePairs, TableLike};
 use crate::{InternalString, Item, KeyMut, RawString, Table, Value};
 
 /// Type representing a TOML inline table,
@@ -11,6 +11,8 @@ use crate::{InternalString, Item, KeyMut, RawString, Table, Value};
 pub struct InlineTable {
     // `preamble` represents whitespaces in an empty table
     preamble: RawString,
+    // Whether to hide an empty table
+    pub(crate) implicit: bool,
     // prefix before `{` and suffix after `}`
     decor: Decor,
     pub(crate) span: Option<std::ops::Range<usize>>,
@@ -55,15 +57,15 @@ impl InlineTable {
         values
     }
 
-    pub(crate) fn append_values<'s, 'c>(
+    pub(crate) fn append_values<'s>(
         &'s self,
         parent: &[&'s Key],
-        values: &'c mut Vec<(Vec<&'s Key>, &'s Value)>,
+        values: &mut Vec<(Vec<&'s Key>, &'s Value)>,
     ) {
-        for value in self.items.values() {
+        for (key, value) in self.items.iter() {
             let mut path = parent.to_vec();
-            path.push(&value.key);
-            match &value.value {
+            path.push(key);
+            match value {
                 Item::Value(Value::InlineTable(table)) if table.is_dotted() => {
                     table.append_values(&path, values);
                 }
@@ -84,8 +86,8 @@ impl InlineTable {
     pub fn sort_values(&mut self) {
         // Assuming standard tables have their position set and this won't negatively impact them
         self.items.sort_keys();
-        for kv in self.items.values_mut() {
-            match &mut kv.value {
+        for value in self.items.values_mut() {
+            match value {
                 Item::Value(Value::InlineTable(table)) if table.is_dotted() => {
                     table.sort_values();
                 }
@@ -109,22 +111,19 @@ impl InlineTable {
     where
         F: FnMut(&Key, &Value, &Key, &Value) -> std::cmp::Ordering,
     {
-        let modified_cmp = |_: &InternalString,
-                            val1: &TableKeyValue,
-                            _: &InternalString,
-                            val2: &TableKeyValue|
-         -> std::cmp::Ordering {
-            match (val1.value.as_value(), val2.value.as_value()) {
-                (Some(v1), Some(v2)) => compare(&val1.key, v1, &val2.key, v2),
-                (Some(_), None) => std::cmp::Ordering::Greater,
-                (None, Some(_)) => std::cmp::Ordering::Less,
-                (None, None) => std::cmp::Ordering::Equal,
-            }
-        };
+        let modified_cmp =
+            |key1: &Key, val1: &Item, key2: &Key, val2: &Item| -> std::cmp::Ordering {
+                match (val1.as_value(), val2.as_value()) {
+                    (Some(v1), Some(v2)) => compare(key1, v1, key2, v2),
+                    (Some(_), None) => std::cmp::Ordering::Greater,
+                    (None, Some(_)) => std::cmp::Ordering::Less,
+                    (None, None) => std::cmp::Ordering::Equal,
+                }
+            };
 
         self.items.sort_by(modified_cmp);
-        for kv in self.items.values_mut() {
-            match &mut kv.value {
+        for value in self.items.values_mut() {
+            match value {
                 Item::Value(Value::InlineTable(table)) if table.is_dotted() => {
                     table.sort_values_by_internal(compare);
                 }
@@ -133,6 +132,36 @@ impl InlineTable {
         }
     }
 
+    /// If a table has no key/value pairs and implicit, it will not be displayed.
+    ///
+    /// # Examples
+    ///
+    /// ```notrust
+    /// [target."x86_64/windows.json".dependencies]
+    /// ```
+    ///
+    /// In the document above, tables `target` and `target."x86_64/windows.json"` are implicit.
+    ///
+    /// ```
+    /// # #[cfg(feature = "parse")] {
+    /// # #[cfg(feature = "display")] {
+    /// use toml_edit::DocumentMut;
+    /// let mut doc = "[a]\n[a.b]\n".parse::<DocumentMut>().expect("invalid toml");
+    ///
+    /// doc["a"].as_table_mut().unwrap().set_implicit(true);
+    /// assert_eq!(doc.to_string(), "[a.b]\n");
+    /// # }
+    /// # }
+    /// ```
+    pub(crate) fn set_implicit(&mut self, implicit: bool) {
+        self.implicit = implicit;
+    }
+
+    /// If a table has no key/value pairs and implicit, it will not be displayed.
+    pub(crate) fn is_implicit(&self) -> bool {
+        self.implicit
+    }
+
     /// Change this table's dotted status
     pub fn set_dotted(&mut self, yes: bool) {
         self.dotted = yes;
@@ -153,14 +182,34 @@ impl InlineTable {
         &self.decor
     }
 
-    /// Returns the decor associated with a given key of the table.
-    pub fn key_decor_mut(&mut self, key: &str) -> Option<&mut Decor> {
-        self.items.get_mut(key).map(|kv| &mut kv.key.decor)
+    /// Returns an accessor to a key's formatting
+    pub fn key(&self, key: &str) -> Option<&'_ Key> {
+        self.items.get_full(key).map(|(_, key, _)| key)
+    }
+
+    /// Returns an accessor to a key's formatting
+    pub fn key_mut(&mut self, key: &str) -> Option<KeyMut<'_>> {
+        use indexmap::map::MutableKeys;
+        self.items
+            .get_full_mut2(key)
+            .map(|(_, key, _)| key.as_mut())
     }
 
     /// Returns the decor associated with a given key of the table.
+    #[deprecated(since = "0.21.1", note = "Replaced with `key_mut`")]
+    pub fn key_decor_mut(&mut self, key: &str) -> Option<&mut Decor> {
+        #![allow(deprecated)]
+        use indexmap::map::MutableKeys;
+        self.items
+            .get_full_mut2(key)
+            .map(|(_, key, _)| key.leaf_decor_mut())
+    }
+
+    /// Returns the decor associated with a given key of the table.
+    #[deprecated(since = "0.21.1", note = "Replaced with `key_mut`")]
     pub fn key_decor(&self, key: &str) -> Option<&Decor> {
-        self.items.get(key).map(|kv| &kv.key.decor)
+        #![allow(deprecated)]
+        self.items.get_full(key).map(|(_, key, _)| key.leaf_decor())
     }
 
     /// Set whitespace after before element
@@ -173,18 +222,21 @@ impl InlineTable {
         &self.preamble
     }
 
-    /// Returns the location within the original document
-    pub(crate) fn span(&self) -> Option<std::ops::Range<usize>> {
+    /// The location within the original document
+    ///
+    /// This generally requires an [`ImDocument`][crate::ImDocument].
+    pub fn span(&self) -> Option<std::ops::Range<usize>> {
         self.span.clone()
     }
 
     pub(crate) fn despan(&mut self, input: &str) {
+        use indexmap::map::MutableKeys;
         self.span = None;
         self.decor.despan(input);
         self.preamble.despan(input);
-        for kv in self.items.values_mut() {
-            kv.key.despan(input);
-            kv.value.despan(input);
+        for (key, value) in self.items.iter_mut2() {
+            key.despan(input);
+            value.despan(input);
         }
     }
 }
@@ -195,18 +247,19 @@ impl InlineTable {
         Box::new(
             self.items
                 .iter()
-                .filter(|&(_, kv)| kv.value.is_value())
-                .map(|(k, kv)| (&k[..], kv.value.as_value().unwrap())),
+                .filter(|(_, value)| !value.is_none())
+                .map(|(key, value)| (key.get(), value.as_value().unwrap())),
         )
     }
 
     /// Returns an iterator over key/value pairs.
     pub fn iter_mut(&mut self) -> InlineTableIterMut<'_> {
+        use indexmap::map::MutableKeys;
         Box::new(
             self.items
-                .iter_mut()
-                .filter(|(_, kv)| kv.value.is_value())
-                .map(|(_, kv)| (kv.key.as_mut(), kv.value.as_value_mut().unwrap())),
+                .iter_mut2()
+                .filter(|(_, value)| value.is_value())
+                .map(|(key, value)| (key.as_mut(), value.as_value_mut().unwrap())),
         )
     }
 
@@ -215,22 +268,22 @@ impl InlineTable {
         self.iter().count()
     }
 
-    /// Returns true iff the table is empty.
+    /// Returns true if the table is empty.
     pub fn is_empty(&self) -> bool {
         self.len() == 0
     }
 
     /// Clears the table, removing all key-value pairs. Keeps the allocated memory for reuse.
     pub fn clear(&mut self) {
-        self.items.clear()
+        self.items.clear();
     }
 
     /// Gets the given key's corresponding entry in the Table for in-place manipulation.
     pub fn entry(&'_ mut self, key: impl Into<InternalString>) -> InlineEntry<'_> {
-        match self.items.entry(key.into()) {
+        match self.items.entry(key.into().into()) {
             indexmap::map::Entry::Occupied(mut entry) => {
                 // Ensure it is a `Value` to simplify `InlineOccupiedEntry`'s code.
-                let scratch = std::mem::take(&mut entry.get_mut().value);
+                let scratch = std::mem::take(entry.get_mut());
                 let scratch = Item::Value(
                     scratch
                         .into_value()
@@ -238,23 +291,21 @@ impl InlineTable {
                         // "safe" value
                         .unwrap_or_else(|_| Value::InlineTable(Default::default())),
                 );
-                entry.get_mut().value = scratch;
+                *entry.get_mut() = scratch;
 
                 InlineEntry::Occupied(InlineOccupiedEntry { entry })
             }
-            indexmap::map::Entry::Vacant(entry) => {
-                InlineEntry::Vacant(InlineVacantEntry { entry, key: None })
-            }
+            indexmap::map::Entry::Vacant(entry) => InlineEntry::Vacant(InlineVacantEntry { entry }),
         }
     }
 
     /// Gets the given key's corresponding entry in the Table for in-place manipulation.
     pub fn entry_format<'a>(&'a mut self, key: &Key) -> InlineEntry<'a> {
         // Accept a `&Key` to be consistent with `entry`
-        match self.items.entry(key.get().into()) {
+        match self.items.entry(key.clone()) {
             indexmap::map::Entry::Occupied(mut entry) => {
                 // Ensure it is a `Value` to simplify `InlineOccupiedEntry`'s code.
-                let scratch = std::mem::take(&mut entry.get_mut().value);
+                let scratch = std::mem::take(entry.get_mut());
                 let scratch = Item::Value(
                     scratch
                         .into_value()
@@ -262,33 +313,30 @@ impl InlineTable {
                         // "safe" value
                         .unwrap_or_else(|_| Value::InlineTable(Default::default())),
                 );
-                entry.get_mut().value = scratch;
+                *entry.get_mut() = scratch;
 
                 InlineEntry::Occupied(InlineOccupiedEntry { entry })
             }
-            indexmap::map::Entry::Vacant(entry) => InlineEntry::Vacant(InlineVacantEntry {
-                entry,
-                key: Some(key.clone()),
-            }),
+            indexmap::map::Entry::Vacant(entry) => InlineEntry::Vacant(InlineVacantEntry { entry }),
         }
     }
     /// Return an optional reference to the value at the given the key.
     pub fn get(&self, key: &str) -> Option<&Value> {
-        self.items.get(key).and_then(|kv| kv.value.as_value())
+        self.items.get(key).and_then(|value| value.as_value())
     }
 
     /// Return an optional mutable reference to the value at the given the key.
     pub fn get_mut(&mut self, key: &str) -> Option<&mut Value> {
         self.items
             .get_mut(key)
-            .and_then(|kv| kv.value.as_value_mut())
+            .and_then(|value| value.as_value_mut())
     }
 
     /// Return references to the key-value pair stored for key, if it is present, else None.
     pub fn get_key_value<'a>(&'a self, key: &str) -> Option<(&'a Key, &'a Item)> {
-        self.items.get(key).and_then(|kv| {
-            if !kv.value.is_none() {
-                Some((&kv.key, &kv.value))
+        self.items.get_full(key).and_then(|(_, key, value)| {
+            if !value.is_none() {
+                Some((key, value))
             } else {
                 None
             }
@@ -297,19 +345,20 @@ impl InlineTable {
 
     /// Return mutable references to the key-value pair stored for key, if it is present, else None.
     pub fn get_key_value_mut<'a>(&'a mut self, key: &str) -> Option<(KeyMut<'a>, &'a mut Item)> {
-        self.items.get_mut(key).and_then(|kv| {
-            if !kv.value.is_none() {
-                Some((kv.key.as_mut(), &mut kv.value))
+        use indexmap::map::MutableKeys;
+        self.items.get_full_mut2(key).and_then(|(_, key, value)| {
+            if !value.is_none() {
+                Some((key.as_mut(), value))
             } else {
                 None
             }
         })
     }
 
-    /// Returns true iff the table contains given key.
+    /// Returns true if the table contains given key.
     pub fn contains_key(&self, key: &str) -> bool {
-        if let Some(kv) = self.items.get(key) {
-            kv.value.is_value()
+        if let Some(value) = self.items.get(key) {
+            value.is_value()
         } else {
             false
         }
@@ -324,44 +373,42 @@ impl InlineTable {
     ) -> &mut Value {
         let key = key.into();
         self.items
-            .entry(key.clone())
-            .or_insert(TableKeyValue::new(Key::new(key), Item::Value(value.into())))
-            .value
+            .entry(Key::new(key))
+            .or_insert(Item::Value(value.into()))
             .as_value_mut()
             .expect("non-value type in inline table")
     }
 
     /// Inserts a key-value pair into the map.
     pub fn insert(&mut self, key: impl Into<InternalString>, value: Value) -> Option<Value> {
-        let key = key.into();
-        let kv = TableKeyValue::new(Key::new(key.clone()), Item::Value(value));
+        let key = Key::new(key.into());
+        let value = Item::Value(value);
         self.items
-            .insert(key, kv)
-            .and_then(|kv| kv.value.into_value().ok())
+            .insert(key, value)
+            .and_then(|old| old.into_value().ok())
     }
 
     /// Inserts a key-value pair into the map.
     pub fn insert_formatted(&mut self, key: &Key, value: Value) -> Option<Value> {
-        let kv = TableKeyValue::new(key.to_owned(), Item::Value(value));
+        let key = key.to_owned();
+        let value = Item::Value(value);
         self.items
-            .insert(InternalString::from(key.get()), kv)
-            .filter(|kv| kv.value.is_value())
-            .map(|kv| kv.value.into_value().unwrap())
+            .insert(key, value)
+            .and_then(|old| old.into_value().ok())
     }
 
     /// Removes an item given the key.
     pub fn remove(&mut self, key: &str) -> Option<Value> {
         self.items
             .shift_remove(key)
-            .and_then(|kv| kv.value.into_value().ok())
+            .and_then(|value| value.into_value().ok())
     }
 
     /// Removes a key from the map, returning the stored key and value if the key was previously in the map.
     pub fn remove_entry(&mut self, key: &str) -> Option<(Key, Value)> {
-        self.items.shift_remove(key).and_then(|kv| {
-            let key = kv.key;
-            kv.value.into_value().ok().map(|value| (key, value))
-        })
+        self.items
+            .shift_remove_entry(key)
+            .and_then(|(key, value)| Some((key, value.into_value().ok()?)))
     }
 
     /// Retains only the elements specified by the `keep` predicate.
@@ -375,17 +422,17 @@ impl InlineTable {
         F: FnMut(&str, &mut Value) -> bool,
     {
         self.items.retain(|key, item| {
-            item.value
-                .as_value_mut()
+            item.as_value_mut()
                 .map(|value| keep(key, value))
                 .unwrap_or(false)
         });
     }
 }
 
+#[cfg(feature = "display")]
 impl std::fmt::Display for InlineTable {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        crate::encode::Encode::encode(self, f, None, ("", ""))
+        crate::encode::encode_table(self, f, None, ("", ""))
     }
 }
 
@@ -394,9 +441,7 @@ impl<K: Into<Key>, V: Into<Value>> Extend<(K, V)> for InlineTable {
         for (key, value) in iter {
             let key = key.into();
             let value = Item::Value(value.into());
-            let value = TableKeyValue::new(key, value);
-            self.items
-                .insert(InternalString::from(value.key.get()), value);
+            self.items.insert(key, value);
         }
     }
 }
@@ -420,8 +465,8 @@ impl IntoIterator for InlineTable {
         Box::new(
             self.items
                 .into_iter()
-                .filter(|(_, kv)| kv.value.is_value())
-                .map(|(k, kv)| (k, kv.value.into_value().unwrap())),
+                .filter(|(_, value)| value.is_value())
+                .map(|(key, value)| (key.into(), value.into_value().unwrap())),
         )
     }
 }
@@ -436,13 +481,15 @@ impl<'s> IntoIterator for &'s InlineTable {
 }
 
 fn decorate_inline_table(table: &mut InlineTable) {
-    for (key_decor, value) in table
+    use indexmap::map::MutableKeys;
+    for (mut key, value) in table
         .items
-        .iter_mut()
-        .filter(|&(_, ref kv)| kv.value.is_value())
-        .map(|(_, kv)| (&mut kv.key.decor, kv.value.as_value_mut().unwrap()))
+        .iter_mut2()
+        .filter(|(_, value)| value.is_value())
+        .map(|(key, value)| (key.as_mut(), value.as_value_mut().unwrap()))
     {
-        key_decor.clear();
+        key.leaf_decor_mut().clear();
+        key.dotted_decor_mut().clear();
         value.decor_mut().clear();
     }
 }
@@ -456,13 +503,14 @@ pub type InlineTableIterMut<'a> = Box<dyn Iterator<Item = (KeyMut<'a>, &'a mut V
 
 impl TableLike for InlineTable {
     fn iter(&self) -> Iter<'_> {
-        Box::new(self.items.iter().map(|(key, kv)| (&key[..], &kv.value)))
+        Box::new(self.items.iter().map(|(key, value)| (key.get(), value)))
     }
     fn iter_mut(&mut self) -> IterMut<'_> {
+        use indexmap::map::MutableKeys;
         Box::new(
             self.items
-                .iter_mut()
-                .map(|(_, kv)| (kv.key.as_mut(), &mut kv.value)),
+                .iter_mut2()
+                .map(|(key, value)| (key.as_mut(), value)),
         )
     }
     fn clear(&mut self) {
@@ -475,7 +523,7 @@ impl TableLike for InlineTable {
                 crate::Entry::Occupied(crate::OccupiedEntry { entry })
             }
             indexmap::map::Entry::Vacant(entry) => {
-                crate::Entry::Vacant(crate::VacantEntry { entry, key: None })
+                crate::Entry::Vacant(crate::VacantEntry { entry })
             }
         }
     }
@@ -485,17 +533,16 @@ impl TableLike for InlineTable {
             indexmap::map::Entry::Occupied(entry) => {
                 crate::Entry::Occupied(crate::OccupiedEntry { entry })
             }
-            indexmap::map::Entry::Vacant(entry) => crate::Entry::Vacant(crate::VacantEntry {
-                entry,
-                key: Some(key.to_owned()),
-            }),
+            indexmap::map::Entry::Vacant(entry) => {
+                crate::Entry::Vacant(crate::VacantEntry { entry })
+            }
         }
     }
     fn get<'s>(&'s self, key: &str) -> Option<&'s Item> {
-        self.items.get(key).map(|kv| &kv.value)
+        self.items.get(key)
     }
     fn get_mut<'s>(&'s mut self, key: &str) -> Option<&'s mut Item> {
-        self.items.get_mut(key).map(|kv| &mut kv.value)
+        self.items.get_mut(key)
     }
     fn get_key_value<'a>(&'a self, key: &str) -> Option<(&'a Key, &'a Item)> {
         self.get_key_value(key)
@@ -518,22 +565,30 @@ impl TableLike for InlineTable {
         self.get_values()
     }
     fn fmt(&mut self) {
-        self.fmt()
+        self.fmt();
     }
     fn sort_values(&mut self) {
-        self.sort_values()
+        self.sort_values();
     }
     fn set_dotted(&mut self, yes: bool) {
-        self.set_dotted(yes)
+        self.set_dotted(yes);
     }
     fn is_dotted(&self) -> bool {
         self.is_dotted()
     }
 
+    fn key(&self, key: &str) -> Option<&'_ Key> {
+        self.key(key)
+    }
+    fn key_mut(&mut self, key: &str) -> Option<KeyMut<'_>> {
+        self.key_mut(key)
+    }
     fn key_decor_mut(&mut self, key: &str) -> Option<&mut Decor> {
+        #![allow(deprecated)]
         self.key_decor_mut(key)
     }
     fn key_decor(&self, key: &str) -> Option<&Decor> {
+        #![allow(deprecated)]
         self.key_decor(key)
     }
 }
@@ -589,7 +644,7 @@ impl<'a> InlineEntry<'a> {
 
 /// A view into a single occupied location in a `IndexMap`.
 pub struct InlineOccupiedEntry<'a> {
-    entry: indexmap::map::OccupiedEntry<'a, InternalString, TableKeyValue>,
+    entry: indexmap::map::OccupiedEntry<'a, Key, Item>,
 }
 
 impl<'a> InlineOccupiedEntry<'a> {
@@ -605,47 +660,46 @@ impl<'a> InlineOccupiedEntry<'a> {
     /// assert_eq!("foo", map.entry("foo").key());
     /// ```
     pub fn key(&self) -> &str {
-        self.entry.key().as_str()
+        self.entry.key().get()
     }
 
     /// Gets a mutable reference to the entry key
     pub fn key_mut(&mut self) -> KeyMut<'_> {
-        self.entry.get_mut().key.as_mut()
+        use indexmap::map::MutableEntryKey;
+        self.entry.key_mut().as_mut()
     }
 
     /// Gets a reference to the value in the entry.
     pub fn get(&self) -> &Value {
-        self.entry.get().value.as_value().unwrap()
+        self.entry.get().as_value().unwrap()
     }
 
     /// Gets a mutable reference to the value in the entry.
     pub fn get_mut(&mut self) -> &mut Value {
-        self.entry.get_mut().value.as_value_mut().unwrap()
+        self.entry.get_mut().as_value_mut().unwrap()
     }
 
-    /// Converts the OccupiedEntry into a mutable reference to the value in the entry
+    /// Converts the `OccupiedEntry` into a mutable reference to the value in the entry
     /// with a lifetime bound to the map itself
     pub fn into_mut(self) -> &'a mut Value {
-        self.entry.into_mut().value.as_value_mut().unwrap()
+        self.entry.into_mut().as_value_mut().unwrap()
     }
 
     /// Sets the value of the entry, and returns the entry's old value
     pub fn insert(&mut self, value: Value) -> Value {
-        let mut value = Item::Value(value);
-        std::mem::swap(&mut value, &mut self.entry.get_mut().value);
-        value.into_value().unwrap()
+        let value = Item::Value(value);
+        self.entry.insert(value).into_value().unwrap()
     }
 
     /// Takes the value out of the entry, and returns it
     pub fn remove(self) -> Value {
-        self.entry.shift_remove().value.into_value().unwrap()
+        self.entry.shift_remove().into_value().unwrap()
     }
 }
 
 /// A view into a single empty location in a `IndexMap`.
 pub struct InlineVacantEntry<'a> {
-    entry: indexmap::map::VacantEntry<'a, InternalString, TableKeyValue>,
-    key: Option<Key>,
+    entry: indexmap::map::VacantEntry<'a, Key, Item>,
 }
 
 impl<'a> InlineVacantEntry<'a> {
@@ -661,19 +715,14 @@ impl<'a> InlineVacantEntry<'a> {
     /// assert_eq!("foo", map.entry("foo").key());
     /// ```
     pub fn key(&self) -> &str {
-        self.entry.key().as_str()
+        self.entry.key().get()
     }
 
-    /// Sets the value of the entry with the VacantEntry's key,
+    /// Sets the value of the entry with the `VacantEntry`'s key,
     /// and returns a mutable reference to it
     pub fn insert(self, value: Value) -> &'a mut Value {
         let entry = self.entry;
-        let key = self.key.unwrap_or_else(|| Key::new(entry.key().as_str()));
         let value = Item::Value(value);
-        entry
-            .insert(TableKeyValue::new(key, value))
-            .value
-            .as_value_mut()
-            .unwrap()
+        entry.insert(value).as_value_mut().unwrap()
     }
 }
