@@ -20,6 +20,7 @@ pub struct Table {
     //
     // `None` for user created tables (can be overridden with `set_position`)
     doc_position: Option<usize>,
+    pub(crate) span: Option<std::ops::Range<usize>>,
     pub(crate) items: KeyValuePairs,
 }
 
@@ -48,8 +49,8 @@ impl Table {
 
     /// Convert to an inline table
     pub fn into_inline_table(mut self) -> InlineTable {
-        for (_, kv) in self.items.iter_mut() {
-            kv.value.make_value();
+        for (_, value) in self.items.iter_mut() {
+            value.make_value();
         }
         let mut t = InlineTable::with_pairs(self.items);
         t.fmt();
@@ -69,15 +70,15 @@ impl Table {
         values
     }
 
-    fn append_values<'s, 'c>(
+    fn append_values<'s>(
         &'s self,
         parent: &[&'s Key],
-        values: &'c mut Vec<(Vec<&'s Key>, &'s Value)>,
+        values: &mut Vec<(Vec<&'s Key>, &'s Value)>,
     ) {
-        for value in self.items.values() {
+        for (key, value) in self.items.iter() {
             let mut path = parent.to_vec();
-            path.push(&value.key);
-            match &value.value {
+            path.push(key);
+            match value {
                 Item::Table(table) if table.is_dotted() => {
                     table.append_values(&path, values);
                 }
@@ -108,8 +109,8 @@ impl Table {
     pub fn sort_values(&mut self) {
         // Assuming standard tables have their doc_position set and this won't negatively impact them
         self.items.sort_keys();
-        for kv in self.items.values_mut() {
-            match &mut kv.value {
+        for value in self.items.values_mut() {
+            match value {
                 Item::Table(table) if table.is_dotted() => {
                     table.sort_values();
                 }
@@ -133,18 +134,15 @@ impl Table {
     where
         F: FnMut(&Key, &Item, &Key, &Item) -> std::cmp::Ordering,
     {
-        let modified_cmp = |_: &InternalString,
-                            val1: &TableKeyValue,
-                            _: &InternalString,
-                            val2: &TableKeyValue|
-         -> std::cmp::Ordering {
-            compare(&val1.key, &val1.value, &val2.key, &val2.value)
-        };
+        let modified_cmp =
+            |key1: &Key, val1: &Item, key2: &Key, val2: &Item| -> std::cmp::Ordering {
+                compare(key1, val1, key2, val2)
+            };
 
         self.items.sort_by(modified_cmp);
 
-        for kv in self.items.values_mut() {
-            match &mut kv.value {
+        for value in self.items.values_mut() {
+            match value {
                 Item::Table(table) if table.is_dotted() => {
                     table.sort_values_by_internal(compare);
                 }
@@ -164,11 +162,15 @@ impl Table {
     /// In the document above, tables `target` and `target."x86_64/windows.json"` are implicit.
     ///
     /// ```
-    /// use toml_edit::Document;
-    /// let mut doc = "[a]\n[a.b]\n".parse::<Document>().expect("invalid toml");
+    /// # #[cfg(feature = "parse")] {
+    /// # #[cfg(feature = "display")] {
+    /// use toml_edit::DocumentMut;
+    /// let mut doc = "[a]\n[a.b]\n".parse::<DocumentMut>().expect("invalid toml");
     ///
     /// doc["a"].as_table_mut().unwrap().set_implicit(true);
     /// assert_eq!(doc.to_string(), "[a.b]\n");
+    /// # }
+    /// # }
     /// ```
     pub fn set_implicit(&mut self, implicit: bool) {
         self.implicit = implicit;
@@ -189,12 +191,12 @@ impl Table {
         self.dotted
     }
 
-    /// Sets the position of the `Table` within the `Document`.
+    /// Sets the position of the `Table` within the [`DocumentMut`][crate::DocumentMut].
     pub fn set_position(&mut self, doc_position: usize) {
         self.doc_position = Some(doc_position);
     }
 
-    /// The position of the `Table` within the `Document`.
+    /// The position of the `Table` within the [`DocumentMut`][crate::DocumentMut].
     ///
     /// Returns `None` if the `Table` was created manually (i.e. not via parsing)
     /// in which case its position is set automatically.  This can be overridden with
@@ -213,14 +215,51 @@ impl Table {
         &self.decor
     }
 
-    /// Returns the decor associated with a given key of the table.
-    pub fn key_decor_mut(&mut self, key: &str) -> Option<&mut Decor> {
-        self.items.get_mut(key).map(|kv| &mut kv.key.decor)
+    /// Returns an accessor to a key's formatting
+    pub fn key(&self, key: &str) -> Option<&'_ Key> {
+        self.items.get_full(key).map(|(_, key, _)| key)
+    }
+
+    /// Returns an accessor to a key's formatting
+    pub fn key_mut(&mut self, key: &str) -> Option<KeyMut<'_>> {
+        use indexmap::map::MutableKeys;
+        self.items
+            .get_full_mut2(key)
+            .map(|(_, key, _)| key.as_mut())
     }
 
     /// Returns the decor associated with a given key of the table.
+    #[deprecated(since = "0.21.1", note = "Replaced with `key_mut`")]
+    pub fn key_decor_mut(&mut self, key: &str) -> Option<&mut Decor> {
+        #![allow(deprecated)]
+        use indexmap::map::MutableKeys;
+        self.items
+            .get_full_mut2(key)
+            .map(|(_, key, _)| key.leaf_decor_mut())
+    }
+
+    /// Returns the decor associated with a given key of the table.
+    #[deprecated(since = "0.21.1", note = "Replaced with `key_mut`")]
     pub fn key_decor(&self, key: &str) -> Option<&Decor> {
-        self.items.get(key).map(|kv| &kv.key.decor)
+        #![allow(deprecated)]
+        self.items.get_full(key).map(|(_, key, _)| key.leaf_decor())
+    }
+
+    /// The location within the original document
+    ///
+    /// This generally requires an [`ImDocument`][crate::ImDocument].
+    pub fn span(&self) -> Option<std::ops::Range<usize>> {
+        self.span.clone()
+    }
+
+    pub(crate) fn despan(&mut self, input: &str) {
+        use indexmap::map::MutableKeys;
+        self.span = None;
+        self.decor.despan(input);
+        for (key, value) in self.items.iter_mut2() {
+            key.despan(input);
+            value.despan(input);
+        }
     }
 }
 
@@ -230,34 +269,35 @@ impl Table {
         Box::new(
             self.items
                 .iter()
-                .filter(|(_, kv)| !kv.value.is_none())
-                .map(|(key, kv)| (&key[..], &kv.value)),
+                .filter(|(_, value)| !value.is_none())
+                .map(|(key, value)| (key.get(), value)),
         )
     }
 
     /// Returns an mutable iterator over all key/value pairs, including empty.
     pub fn iter_mut(&mut self) -> IterMut<'_> {
+        use indexmap::map::MutableKeys;
         Box::new(
             self.items
-                .iter_mut()
-                .filter(|(_, kv)| !kv.value.is_none())
-                .map(|(_, kv)| (kv.key.as_mut(), &mut kv.value)),
+                .iter_mut2()
+                .filter(|(_, value)| !value.is_none())
+                .map(|(key, value)| (key.as_mut(), value)),
         )
     }
 
     /// Returns the number of non-empty items in the table.
     pub fn len(&self) -> usize {
-        self.items.iter().filter(|i| !(i.1).value.is_none()).count()
+        self.iter().count()
     }
 
-    /// Returns true iff the table is empty.
+    /// Returns true if the table is empty.
     pub fn is_empty(&self) -> bool {
         self.len() == 0
     }
 
     /// Clears the table, removing all key-value pairs. Keeps the allocated memory for reuse.
     pub fn clear(&mut self) {
-        self.items.clear()
+        self.items.clear();
     }
 
     /// Gets the given key's corresponding entry in the Table for in-place manipulation.
@@ -265,75 +305,87 @@ impl Table {
         // Accept a `&str` rather than an owned type to keep `InternalString`, well, internal
         match self.items.entry(key.into()) {
             indexmap::map::Entry::Occupied(entry) => Entry::Occupied(OccupiedEntry { entry }),
-            indexmap::map::Entry::Vacant(entry) => Entry::Vacant(VacantEntry { entry, key: None }),
+            indexmap::map::Entry::Vacant(entry) => Entry::Vacant(VacantEntry { entry }),
         }
     }
 
     /// Gets the given key's corresponding entry in the Table for in-place manipulation.
     pub fn entry_format<'a>(&'a mut self, key: &Key) -> Entry<'a> {
         // Accept a `&Key` to be consistent with `entry`
-        match self.items.entry(key.get().into()) {
+        match self.items.entry(key.clone()) {
             indexmap::map::Entry::Occupied(entry) => Entry::Occupied(OccupiedEntry { entry }),
-            indexmap::map::Entry::Vacant(entry) => Entry::Vacant(VacantEntry {
-                entry,
-                key: Some(key.to_owned()),
-            }),
+            indexmap::map::Entry::Vacant(entry) => Entry::Vacant(VacantEntry { entry }),
         }
     }
 
     /// Returns an optional reference to an item given the key.
     pub fn get<'a>(&'a self, key: &str) -> Option<&'a Item> {
-        self.items.get(key).and_then(|kv| {
-            if !kv.value.is_none() {
-                Some(&kv.value)
-            } else {
-                None
-            }
-        })
+        self.items
+            .get(key)
+            .and_then(|value| if !value.is_none() { Some(value) } else { None })
     }
 
     /// Returns an optional mutable reference to an item given the key.
     pub fn get_mut<'a>(&'a mut self, key: &str) -> Option<&'a mut Item> {
-        self.items.get_mut(key).and_then(|kv| {
-            if !kv.value.is_none() {
-                Some(&mut kv.value)
+        self.items
+            .get_mut(key)
+            .and_then(|value| if !value.is_none() { Some(value) } else { None })
+    }
+
+    /// Return references to the key-value pair stored for key, if it is present, else None.
+    pub fn get_key_value<'a>(&'a self, key: &str) -> Option<(&'a Key, &'a Item)> {
+        self.items.get_full(key).and_then(|(_, key, value)| {
+            if !value.is_none() {
+                Some((key, value))
             } else {
                 None
             }
         })
     }
 
-    /// Returns true iff the table contains an item with the given key.
+    /// Return mutable references to the key-value pair stored for key, if it is present, else None.
+    pub fn get_key_value_mut<'a>(&'a mut self, key: &str) -> Option<(KeyMut<'a>, &'a mut Item)> {
+        use indexmap::map::MutableKeys;
+        self.items.get_full_mut2(key).and_then(|(_, key, value)| {
+            if !value.is_none() {
+                Some((key.as_mut(), value))
+            } else {
+                None
+            }
+        })
+    }
+
+    /// Returns true if the table contains an item with the given key.
     pub fn contains_key(&self, key: &str) -> bool {
-        if let Some(kv) = self.items.get(key) {
-            !kv.value.is_none()
+        if let Some(value) = self.items.get(key) {
+            !value.is_none()
         } else {
             false
         }
     }
 
-    /// Returns true iff the table contains a table with the given key.
+    /// Returns true if the table contains a table with the given key.
     pub fn contains_table(&self, key: &str) -> bool {
-        if let Some(kv) = self.items.get(key) {
-            kv.value.is_table()
+        if let Some(value) = self.items.get(key) {
+            value.is_table()
         } else {
             false
         }
     }
 
-    /// Returns true iff the table contains a value with the given key.
+    /// Returns true if the table contains a value with the given key.
     pub fn contains_value(&self, key: &str) -> bool {
-        if let Some(kv) = self.items.get(key) {
-            kv.value.is_value()
+        if let Some(value) = self.items.get(key) {
+            value.is_value()
         } else {
             false
         }
     }
 
-    /// Returns true iff the table contains an array of tables with the given key.
+    /// Returns true if the table contains an array of tables with the given key.
     pub fn contains_array_of_tables(&self, key: &str) -> bool {
-        if let Some(kv) = self.items.get(key) {
-            kv.value.is_array_of_tables()
+        if let Some(value) = self.items.get(key) {
+            value.is_array_of_tables()
         } else {
             false
         }
@@ -341,54 +393,66 @@ impl Table {
 
     /// Inserts a key-value pair into the map.
     pub fn insert(&mut self, key: &str, item: Item) -> Option<Item> {
-        let kv = TableKeyValue::new(Key::new(key), item);
-        self.items.insert(key.into(), kv).map(|kv| kv.value)
+        let key = Key::new(key);
+        self.items.insert(key, item)
     }
 
     /// Inserts a key-value pair into the map.
     pub fn insert_formatted(&mut self, key: &Key, item: Item) -> Option<Item> {
-        let kv = TableKeyValue::new(key.to_owned(), item);
-        self.items.insert(key.get().into(), kv).map(|kv| kv.value)
+        let key = key.to_owned();
+        self.items.insert(key, item)
     }
 
     /// Removes an item given the key.
     pub fn remove(&mut self, key: &str) -> Option<Item> {
-        self.items.shift_remove(key).map(|kv| kv.value)
+        self.items.shift_remove(key)
     }
 
     /// Removes a key from the map, returning the stored key and value if the key was previously in the map.
     pub fn remove_entry(&mut self, key: &str) -> Option<(Key, Item)> {
-        self.items.shift_remove(key).map(|kv| (kv.key, kv.value))
+        self.items.shift_remove_entry(key)
+    }
+
+    /// Retains only the elements specified by the `keep` predicate.
+    ///
+    /// In other words, remove all pairs `(key, item)` for which
+    /// `keep(&key, &mut item)` returns `false`.
+    ///
+    /// The elements are visited in iteration order.
+    pub fn retain<F>(&mut self, mut keep: F)
+    where
+        F: FnMut(&str, &mut Item) -> bool,
+    {
+        self.items.retain(|key, value| keep(key, value));
     }
 }
 
+#[cfg(feature = "display")]
 impl std::fmt::Display for Table {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        use crate::encode::Encode;
         let children = self.get_values();
         // print table body
         for (key_path, value) in children {
-            key_path.as_slice().encode(f, DEFAULT_KEY_DECOR)?;
+            crate::encode::encode_key_path_ref(&key_path, f, None, DEFAULT_KEY_DECOR)?;
             write!(f, "=")?;
-            value.encode(f, DEFAULT_VALUE_DECOR)?;
+            crate::encode::encode_value(value, f, None, DEFAULT_VALUE_DECOR)?;
             writeln!(f)?;
         }
         Ok(())
     }
 }
 
-impl<K: Into<Key>, V: Into<Value>> Extend<(K, V)> for Table {
+impl<K: Into<Key>, V: Into<Item>> Extend<(K, V)> for Table {
     fn extend<T: IntoIterator<Item = (K, V)>>(&mut self, iter: T) {
         for (key, value) in iter {
             let key = key.into();
-            let value = Item::Value(value.into());
-            let value = TableKeyValue::new(key, value);
-            self.items.insert(value.key.get().into(), value);
+            let value = value.into();
+            self.items.insert(key, value);
         }
     }
 }
 
-impl<K: Into<Key>, V: Into<Value>> FromIterator<(K, V)> for Table {
+impl<K: Into<Key>, V: Into<Item>> FromIterator<(K, V)> for Table {
     fn from_iter<I>(iter: I) -> Self
     where
         I: IntoIterator<Item = (K, V)>,
@@ -404,7 +468,7 @@ impl IntoIterator for Table {
     type IntoIter = IntoIter;
 
     fn into_iter(self) -> Self::IntoIter {
-        Box::new(self.items.into_iter().map(|(k, kv)| (k, kv.value)))
+        Box::new(self.items.into_iter().map(|(k, value)| (k.into(), value)))
     }
 }
 
@@ -417,37 +481,28 @@ impl<'s> IntoIterator for &'s Table {
     }
 }
 
-pub(crate) type KeyValuePairs = IndexMap<InternalString, TableKeyValue>;
+pub(crate) type KeyValuePairs = IndexMap<Key, Item>;
 
 fn decorate_table(table: &mut Table) {
-    for (key_decor, value) in table
+    use indexmap::map::MutableKeys;
+    for (mut key, value) in table
         .items
-        .iter_mut()
-        .filter(|&(_, ref kv)| kv.value.is_value())
-        .map(|(_, kv)| (&mut kv.key.decor, kv.value.as_value_mut().unwrap()))
+        .iter_mut2()
+        .filter(|(_, value)| value.is_value())
+        .map(|(key, value)| (key.as_mut(), value.as_value_mut().unwrap()))
     {
-        key_decor.clear();
+        key.leaf_decor_mut().clear();
+        key.dotted_decor_mut().clear();
         value.decor_mut().clear();
     }
 }
 
 // `key1 = value1`
+pub(crate) const DEFAULT_ROOT_DECOR: (&str, &str) = ("", "");
 pub(crate) const DEFAULT_KEY_DECOR: (&str, &str) = ("", " ");
 pub(crate) const DEFAULT_TABLE_DECOR: (&str, &str) = ("\n", "");
 pub(crate) const DEFAULT_KEY_PATH_DECOR: (&str, &str) = ("", "");
 
-#[derive(Debug, Clone)]
-pub(crate) struct TableKeyValue {
-    pub(crate) key: Key,
-    pub(crate) value: Item,
-}
-
-impl TableKeyValue {
-    pub(crate) fn new(key: Key, value: Item) -> Self {
-        TableKeyValue { key, value }
-    }
-}
-
 /// An owned iterator type over `Table`'s key/value pairs.
 pub type IntoIter = Box<dyn Iterator<Item = (InternalString, Item)>>;
 /// An iterator type over `Table`'s key/value pairs.
@@ -465,15 +520,25 @@ pub trait TableLike: crate::private::Sealed {
     fn len(&self) -> usize {
         self.iter().filter(|&(_, v)| !v.is_none()).count()
     }
-    /// Returns true iff the table is empty.
+    /// Returns true if the table is empty.
     fn is_empty(&self) -> bool {
         self.len() == 0
     }
+    /// Clears the table, removing all key-value pairs. Keeps the allocated memory for reuse.
+    fn clear(&mut self);
+    /// Gets the given key's corresponding entry in the Table for in-place manipulation.
+    fn entry<'a>(&'a mut self, key: &str) -> Entry<'a>;
+    /// Gets the given key's corresponding entry in the Table for in-place manipulation.
+    fn entry_format<'a>(&'a mut self, key: &Key) -> Entry<'a>;
     /// Returns an optional reference to an item given the key.
     fn get<'s>(&'s self, key: &str) -> Option<&'s Item>;
     /// Returns an optional mutable reference to an item given the key.
     fn get_mut<'s>(&'s mut self, key: &str) -> Option<&'s mut Item>;
-    /// Returns true iff the table contains an item with the given key.
+    /// Return references to the key-value pair stored for key, if it is present, else None.
+    fn get_key_value<'a>(&'a self, key: &str) -> Option<(&'a Key, &'a Item)>;
+    /// Return mutable references to the key-value pair stored for key, if it is present, else None.
+    fn get_key_value_mut<'a>(&'a mut self, key: &str) -> Option<(KeyMut<'a>, &'a mut Item)>;
+    /// Returns true if the table contains an item with the given key.
     fn contains_key(&self, key: &str) -> bool;
     /// Inserts a key-value pair into the map.
     fn insert(&mut self, key: &str, value: Item) -> Option<Item>;
@@ -496,9 +561,15 @@ pub trait TableLike: crate::private::Sealed {
     /// Check if this is a wrapper for dotted keys, rather than a standard table
     fn is_dotted(&self) -> bool;
 
+    /// Returns an accessor to a key's formatting
+    fn key(&self, key: &str) -> Option<&'_ Key>;
+    /// Returns an accessor to a key's formatting
+    fn key_mut(&mut self, key: &str) -> Option<KeyMut<'_>>;
     /// Returns the decor associated with a given key of the table.
+    #[deprecated(since = "0.21.1", note = "Replaced with `key_mut`")]
     fn key_decor_mut(&mut self, key: &str) -> Option<&mut Decor>;
     /// Returns the decor associated with a given key of the table.
+    #[deprecated(since = "0.21.1", note = "Replaced with `key_mut`")]
     fn key_decor(&self, key: &str) -> Option<&Decor>;
 }
 
@@ -509,12 +580,27 @@ impl TableLike for Table {
     fn iter_mut(&mut self) -> IterMut<'_> {
         self.iter_mut()
     }
+    fn clear(&mut self) {
+        self.clear();
+    }
+    fn entry<'a>(&'a mut self, key: &str) -> Entry<'a> {
+        self.entry(key)
+    }
+    fn entry_format<'a>(&'a mut self, key: &Key) -> Entry<'a> {
+        self.entry_format(key)
+    }
     fn get<'s>(&'s self, key: &str) -> Option<&'s Item> {
         self.get(key)
     }
     fn get_mut<'s>(&'s mut self, key: &str) -> Option<&'s mut Item> {
         self.get_mut(key)
     }
+    fn get_key_value<'a>(&'a self, key: &str) -> Option<(&'a Key, &'a Item)> {
+        self.get_key_value(key)
+    }
+    fn get_key_value_mut<'a>(&'a mut self, key: &str) -> Option<(KeyMut<'a>, &'a mut Item)> {
+        self.get_key_value_mut(key)
+    }
     fn contains_key(&self, key: &str) -> bool {
         self.contains_key(key)
     }
@@ -529,22 +615,30 @@ impl TableLike for Table {
         self.get_values()
     }
     fn fmt(&mut self) {
-        self.fmt()
+        self.fmt();
     }
     fn sort_values(&mut self) {
-        self.sort_values()
+        self.sort_values();
     }
     fn is_dotted(&self) -> bool {
         self.is_dotted()
     }
     fn set_dotted(&mut self, yes: bool) {
-        self.set_dotted(yes)
+        self.set_dotted(yes);
     }
 
+    fn key(&self, key: &str) -> Option<&'_ Key> {
+        self.key(key)
+    }
+    fn key_mut(&mut self, key: &str) -> Option<KeyMut<'_>> {
+        self.key_mut(key)
+    }
     fn key_decor_mut(&mut self, key: &str) -> Option<&mut Decor> {
+        #![allow(deprecated)]
         self.key_decor_mut(key)
     }
     fn key_decor(&self, key: &str) -> Option<&Decor> {
+        #![allow(deprecated)]
         self.key_decor(key)
     }
 }
@@ -597,7 +691,7 @@ impl<'a> Entry<'a> {
 
 /// A view into a single occupied location in a `IndexMap`.
 pub struct OccupiedEntry<'a> {
-    entry: indexmap::map::OccupiedEntry<'a, InternalString, TableKeyValue>,
+    pub(crate) entry: indexmap::map::OccupiedEntry<'a, Key, Item>,
 }
 
 impl<'a> OccupiedEntry<'a> {
@@ -613,41 +707,45 @@ impl<'a> OccupiedEntry<'a> {
     /// assert_eq!("foo", map.entry("foo").key());
     /// ```
     pub fn key(&self) -> &str {
-        self.entry.key().as_str()
+        self.entry.key().get()
+    }
+
+    /// Gets a mutable reference to the entry key
+    pub fn key_mut(&mut self) -> KeyMut<'_> {
+        use indexmap::map::MutableEntryKey;
+        self.entry.key_mut().as_mut()
     }
 
     /// Gets a reference to the value in the entry.
     pub fn get(&self) -> &Item {
-        &self.entry.get().value
+        self.entry.get()
     }
 
     /// Gets a mutable reference to the value in the entry.
     pub fn get_mut(&mut self) -> &mut Item {
-        &mut self.entry.get_mut().value
+        self.entry.get_mut()
     }
 
-    /// Converts the OccupiedEntry into a mutable reference to the value in the entry
+    /// Converts the `OccupiedEntry` into a mutable reference to the value in the entry
     /// with a lifetime bound to the map itself
     pub fn into_mut(self) -> &'a mut Item {
-        &mut self.entry.into_mut().value
+        self.entry.into_mut()
     }
 
     /// Sets the value of the entry, and returns the entry's old value
-    pub fn insert(&mut self, mut value: Item) -> Item {
-        std::mem::swap(&mut value, &mut self.entry.get_mut().value);
-        value
+    pub fn insert(&mut self, value: Item) -> Item {
+        self.entry.insert(value)
     }
 
     /// Takes the value out of the entry, and returns it
     pub fn remove(self) -> Item {
-        self.entry.shift_remove().value
+        self.entry.shift_remove()
     }
 }
 
 /// A view into a single empty location in a `IndexMap`.
 pub struct VacantEntry<'a> {
-    entry: indexmap::map::VacantEntry<'a, InternalString, TableKeyValue>,
-    key: Option<Key>,
+    pub(crate) entry: indexmap::map::VacantEntry<'a, Key, Item>,
 }
 
 impl<'a> VacantEntry<'a> {
@@ -663,14 +761,13 @@ impl<'a> VacantEntry<'a> {
     /// assert_eq!("foo", map.entry("foo").key());
     /// ```
     pub fn key(&self) -> &str {
-        self.entry.key().as_str()
+        self.entry.key().get()
     }
 
-    /// Sets the value of the entry with the VacantEntry's key,
+    /// Sets the value of the entry with the `VacantEntry`'s key,
     /// and returns a mutable reference to it
     pub fn insert(self, value: Item) -> &'a mut Item {
         let entry = self.entry;
-        let key = self.key.unwrap_or_else(|| Key::new(entry.key().as_str()));
-        &mut entry.insert(TableKeyValue::new(key, value)).value
+        entry.insert(value)
     }
 }
